// Cartesian.H,v 0.3
//
// Copyright 2000 by Roman Kantor.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// version 2 as published by the Free Software Foundation.
//
// This library is distributed  WITHOUT ANY WARRANTY;
// WITHOUT even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


#include "Cartesian.h"
#include <stdio.h>
#include <string.h>


const int        CANVAS_BORDER = 5;                //gap between the graphics and surrounding"box"
const int        AXIS_BORDER = 3;                //gap between axis drawing(i.e.axis line) and its "box"
const int        MINOR_INTERVAL = 0;                //0 stands for automatic choice in default_*_intervals array
const int        MINOR_SEPARATION = 13;            
const int        MAJOR_STEP = 5;
const int        LABEL_STEP = 10;
const int        LABEL_SIZE = 14;
const Fl_Font    LABEL_FONT = FL_HELVETICA;




const int MAX_LABEL_FORMAT = 16;
const int MAX_LABEL_LENGTH = 32;

const int NO_LIN_DEFAULTS=3;
double default_lin_intervals[NO_LIN_DEFAULTS]=    {1,    2,    5};
int default_lin_major_steps[NO_LIN_DEFAULTS]=    {5,    5,    2};
int default_lin_label_steps[NO_LIN_DEFAULTS]=    {10,5,    4};

const int NO_LOG_DEFAULTS = 3;
double default_log_intervals[NO_LOG_DEFAULTS]=    {1,    2,    5};
int default_log_major_steps[NO_LOG_DEFAULTS]=    {5,    5,    2};
int default_log_label_steps[NO_LOG_DEFAULTS]=    {10,5,    2};

////////////////////    Ca_Axis    ////////////////////////////

void Ca_Axis_::min(double x){
    min_=x;
    if(!valid_){
        max_=x;
        valid_=1;
    }
    damage(CA_DAMAGE_ALL);
    canvas_->damage(CA_DAMAGE_ALL);

};

void Ca_Axis_::max(double x){
    max_=x;
    if(!valid_){
        min_=x;
        valid_=1;
    }
    damage(CA_DAMAGE_ALL);
    canvas_->damage(CA_DAMAGE_ALL);

};

Ca_Axis_::Ca_Axis_(int x, int y, int w, int h, const char * label):Fl_Box(x,y,w,h,label)
{    
    box(FL_FLAT_BOX);
    canvas_=Ca_Canvas::current();
    previous_axis_=canvas_->last_axis_;
    canvas_->last_axis_=this;
    tick_interval(MINOR_INTERVAL);
    tick_separation(MINOR_SEPARATION);
    major_step(MAJOR_STEP);
    label_step(LABEL_STEP);
    border(AXIS_BORDER);
    axis_color_=FL_BLACK;
    label_font(FL_HELVETICA);
    label_size(LABEL_SIZE);
    label_format(0);
    minor_grid_color_=major_grid_color_=label_grid_color_=FL_BLACK;
    label_grid_width_=major_grid_width_=minor_grid_width_=0;
    minor_grid_style_=major_grid_style_=label_grid_style_=FL_SOLID;
    minor_grid_dashes_= major_grid_dashes_=label_grid_dashes_=0;
    grid_visible_=0;
    min_=max_=0;
    min_pos_=max_pos_=0;
    valid_=0;
    tick_length_=0;

};

Ca_Axis_::~Ca_Axis_(){

    if (canvas_->last_axis_==this)
        canvas_->last_axis_=previous_axis_;
    else{
        Ca_Axis_ *axis=canvas_->last_axis_;
        while(axis)
            if(axis->previous_axis_==this){
                axis->previous_axis_=previous_axis_;
                break;
            }
    };
};



void Ca_Axis_::rescale(int when, double  x){
    if(!valid_){
        max_=x;
        min_=x;
        damage(CA_DAMAGE_ALL);
        canvas_->damage(CA_DAMAGE_ALL);
        valid_=1;
        return;
    }

    if((when&CA_WHEN_MAX)&&(x>max_)){
        max_=x;
        damage(CA_DAMAGE_ALL);
        canvas_->damage(CA_DAMAGE_ALL);
    }
    if((when&CA_WHEN_MIN)&&(x<min_)){
        min_=x;
        damage(CA_DAMAGE_ALL);
        canvas_->damage(CA_DAMAGE_ALL);
    }
};

//////////////////////////////////////////////////////////////////////

void Ca_X_Axis_::draw(){
    int tick_index=-1;
    double tick_value;
    int tick_order;//, tick_number;
    double _interval=0;
    const char * label_format=label_format_;
    if(damage()|FL_DAMAGE_ALL)
        draw_label();
    if (damage()&(FL_DAMAGE_ALL|CA_DAMAGE_ALL)){
        update();
        if (box()==FL_NO_BOX){
            fl_color(parent()->color());
            fl_rectf(x(),y(),w(),h());
        }else
            draw_box();
        if(!valid_) return;
        fl_font(label_font_face_,label_font_size_);
        int a, b, l1, l2, m1, m2, l, _pos,_x,_y,_w,_h; //temporary coordinates for ticks
        fl_color(axis_color_);
        fl_clip(x()+border_,y()+border_,w()-2*border_,h()-2*border_);
        a=y()+border_;
        b=y()+h()-border_;
        switch(axis_align_ & CA_ALIGNMENT){
            case CA_BOTTOM:
                l=l1=m1=a;
                if (tick_length_)
                    m2=m1+tick_length_;
                else
                    m2=m1+label_font_size_;
                l2=(m1+m2)/2;
                break;
            case CA_TOP:
                l=l2=m2=b-1;
                if (tick_length_)
                    m1=m2-tick_length_;
                else
                    m1=m2-label_font_size_;
                l1=(m1+m2)/2;
                break;
            case CA_CENTER:
                m1=a;
                m2=b;
                l=(a+b)/2;
                l1=(a+l)/2;
                l2=(l+b)/2;
                break;
        }
        if(axis_align_ & CA_LINE)
            fl_xyline(x()+border_,l,x()+w()-border_);
//        tick_number=0;
        while(next_tick(tick_index, tick_value, tick_order, _interval)){
            _pos=position(tick_value);
            if(tick_index % major_step_)
                fl_yxline(_pos,l1,l2);
            else
                fl_yxline(_pos,m1,m2);
                
            if(!(tick_index % label_step_)){
                char label[MAX_LABEL_LENGTH];
                char _label_format[MAX_LABEL_FORMAT];
                if(!label_format){
                    int _tick_order;
                    if (tick_order>=0)
                        _tick_order=0;
                    else
                        _tick_order=-tick_order;
                    sprintf(_label_format,"%s.%if","%",_tick_order);
                }
                else
                    strcpy(_label_format,label_format);
                sprintf(label, _label_format,tick_value);
                fl_measure(label,_w,_h);
                _x=_pos-_w/2;
                switch (axis_align_ & CA_ALIGNMENT){
                    case CA_TOP:
                        _y=m1-_h/3;
                        break;
                    case CA_BOTTOM:
                        _y=m2+_h;
                        break;
                    case CA_CENTER:
                        _y=l+_h/3;
                        Fl_Color _color=fl_color();
                        fl_color(color());
                        fl_rectf(_x-_h/6,l-_h/2,_w+_h/3,_h);
                        fl_color(_color);
                        break;
                }
                fl_draw(label,_x,_y);
            }
        }
        fl_pop_clip();
    }
};

/////////////////////////////////////////////////////////////////////////////////////////////////////
void Ca_X_Axis_::current(){
    canvas_->current_x(this);
};

void Ca_X_Axis_::draw_grid(){
    if((!grid_visible_)||(!valid_))return;
    int tick_index=-1;
    double tick_value;
    int tick_order;
    double _interval=0;
    int l1,l2;
    l1=canvas_->y()+canvas_->border();
    l2=canvas_->y()+canvas_->h()-canvas_->border();
    while(next_tick(tick_index, tick_value, tick_order, _interval)){
        if((grid_visible_ & CA_LABEL_GRID) && !(tick_index % label_step_)){
            fl_color(label_grid_color_);
#ifdef LINESTYLES
            fl_line_style(label_grid_style_,label_grid_width_);
#endif
            fl_yxline(position(tick_value),l1,l2);
#ifdef LINESTYLES
            fl_line_style(FL_SOLID);
#endif
        }else if((grid_visible_ & CA_MAJOR_GRID) && !(tick_index % major_step_)){
            fl_color(major_grid_color_);
#ifdef LINESTYLES
            fl_line_style(major_grid_style_,major_grid_width_);
#endif
            fl_yxline(position(tick_value),l1,l2);
#ifdef LINESTYLES
            fl_line_style(FL_SOLID);
#endif
        }else if((grid_visible_&CA_MINOR_GRID)){
            fl_color(minor_grid_color_);
#ifdef LINESTYLES
            fl_line_style(minor_grid_style_,minor_grid_width_);
#endif
            fl_yxline(position(tick_value),l1,l2);
#ifdef LINESTYLES
            fl_line_style(FL_SOLID);
#endif
        }
    }
    
};



int Ca_X_Axis_::give_objects(Ca_Axis_ *axis){
    int n=0;
    Ca_ObjectChain *ochain=canvas_->first_object_;
    Ca_ObjectChain *next;
    while (ochain){
        next=ochain->next;
        if(ochain->object->x_axis_==this){
            if (axis)
                ochain->object->x_axis_=axis;
            else
                delete ochain->object;
            n++;
        }
        ochain=next;
    }
    return n;
};



Ca_X_Axis_::Ca_X_Axis_(int x, int y, int w, int h, const char *label):Ca_Axis_(x, y, w,  h,  label){
    if(!(canvas_->current_x()))
        current();
    axis_align(CA_BOTTOM|CA_LINE);
};    

void Ca_Y_Axis_::draw(){
    int tick_index=-1;
    double tick_value;
    int tick_order;//,tick_number;
    double _interval=0;
    const char * label_format=label_format_;
    if(damage()|FL_DAMAGE_ALL)
        draw_label();
    if (damage()&(FL_DAMAGE_ALL|CA_DAMAGE_ALL)){
        update();
        if (box()==FL_NO_BOX){
            fl_color(parent()->color());
            fl_rectf(x(),y(),w(),h());
        }else
            draw_box();
        if(!valid_) return;
        fl_font(label_font_face_,label_font_size_);
        int a, b, l1, l2, m1, m2, l, _pos,_x,_y,_w,_h; //temporary coordinates for ticks
        fl_color(axis_color_);;
        fl_clip(x()+border_,y()+border_,w()-2*border_,h()-2*border_);
        a=x()+border_;
        b=x()+w()-border_;
        switch(axis_align_ & CA_ALIGNMENT){
            case CA_RIGHT:
                l=l1=m1=a;
                if (tick_length_)
                    m2=m1+tick_length_;
                else
                    m2=m1+label_font_size_;
                l2=(l1+m2)/2;
                break;
            case CA_LEFT:
                l=l2=m2=b-1;
                if (tick_length_)
                    m1=m2-tick_length_;
                else
                    m1=m2-label_font_size_;
                l1=(m1+m2)/2;
                break;
            case CA_CENTER:
                m1=a;
                m2=b;
                l=(a+b)/2;
                l1=(a+l)/2;
                l2=(l+b)/2;
                break;
        }
        if(axis_align_ & CA_LINE)
            fl_yxline(l,y()+border_,y()+h()-border_);
        while(next_tick(tick_index, tick_value, tick_order, _interval)){
            _pos=position(tick_value);
            if(tick_index % major_step_)
                fl_xyline(l1,_pos, l2);
            else
                fl_xyline(m1,_pos, m2);
            if(!(tick_index % label_step_)){
                char label[MAX_LABEL_LENGTH];
                char _label_format[MAX_LABEL_FORMAT];
                if(!label_format){
                    int _tick_order;
                    if (tick_order>=0)
                        _tick_order=0;
                    else
                        _tick_order=-tick_order;
                    sprintf(_label_format,"%s.%if","%",_tick_order);
                }
                else
                    strcpy(_label_format,label_format);
                sprintf(label, _label_format,tick_value);
                fl_measure(label,_w,_h);
                _y=_pos+_h/3;
                switch (axis_align_ & CA_ALIGNMENT){
                    case CA_LEFT:
                        _x=m1-_h/3-_w;
                        break;
                    case CA_RIGHT:
                        _x=m2+_h/3;
                        break;
                    case CA_CENTER:
                        _x=(m1+m2)/2-_w/2;
                        Fl_Color _color=fl_color();
                        fl_color(color());
                        fl_rectf(_x-_h/6,_pos-_h/2,_w+_h/3,_h);
                        fl_color(_color);
                        break;
                }
                fl_draw(label,_x,_y);
            }
        }
        fl_pop_clip();
    }
};


void Ca_Y_Axis_::current(){
    canvas_->current_y(this);
};

    

void Ca_Y_Axis_::draw_grid(){
    if(!(grid_visible_)||(!valid_))return;
    int tick_index=-1;
    double tick_value;
    int tick_order;
    double _interval=0;
    int l1,l2;
    l1=canvas_->x()+canvas_->border();
    l2=canvas_->x()+canvas_->w()-canvas_->border();
    while(next_tick(tick_index, tick_value, tick_order,_interval)){
        if((grid_visible_&CA_LABEL_GRID) && !(tick_index % label_step_)){
            fl_color(label_grid_color_);
#ifdef LINESTYLES
            fl_line_style(label_grid_style_,label_grid_width_);
#endif
            fl_xyline(l1,position(tick_value),l2);
#ifdef LINESTYLES
            fl_line_style(FL_SOLID);
#endif
        }else if((grid_visible_&CA_MAJOR_GRID) && !(tick_index % major_step_)){
            fl_color(major_grid_color_);
#ifdef LINESTYLES
            fl_line_style(major_grid_style_,major_grid_width_);
#endif
            fl_xyline(l1,position(tick_value),l2);
#ifdef LINESTYLES
            fl_line_style(FL_SOLID);
#endif
        }else if(grid_visible_&CA_MINOR_GRID){
            fl_color(minor_grid_color_);
#ifdef LINESTYLES
            fl_line_style(minor_grid_style_,minor_grid_width_);
#endif
            fl_xyline(l1,position(tick_value),l2);
#ifdef LINESTYLES
            fl_line_style(FL_SOLID);
#endif
        }
    }
};


int Ca_Y_Axis_::give_objects(Ca_Axis_ *axis){
    int n=0;
    Ca_ObjectChain *ochain=canvas_->first_object_;
    Ca_ObjectChain *next;
    while (ochain){
        next=ochain->next;
        if(ochain->object->y_axis_==this){
            if (axis)
                ochain->object->y_axis_=axis;
            else
                delete ochain->object;
            n++;
        }
        ochain=next;
    }
    return n;
};

    
    
Ca_Y_Axis_::Ca_Y_Axis_(int x, int y, int w, int h, const char * label):Ca_Axis_(x, y, w,  h,  label){
    if(!(canvas_->current_x()))
        current();
    axis_align(CA_LEFT|CA_LINE);
};





Ca_RevX_Axis_::Ca_RevX_Axis_(int x, int y, int w, int h, const char * label):Ca_Axis_(x,y,w,h,label),Ca_X_Axis_(x,y,w,h,label){};

Ca_RevY_Axis_::Ca_RevY_Axis_(int x, int y, int w, int h, const char * label):Ca_Axis_(x,y,w,h,label),Ca_Y_Axis_(x, y, w,  h,  label){};


///////////////////////////////////////////////////////////////////////////////

Ca_Canvas *Ca_Canvas::current_=0;

void Ca_Canvas::draw(){
    
    uchar damage_= damage();
    int _b=border_/2;
    int _x=x()+_b;
    int _y=y()+_b;
    int _w=w()-2*_b;
    int _h=h()-2*_b;
    int replot=0;

    Ca_Axis_ *axis = last_axis_;

///{ something similar will go in the future into the lauout layer...
    while (axis){
        replot |= axis->update();
        axis=axis->previous_axis_;
    }
///}

    if((damage_!=CA_DAMAGE_ADD)||replot){
        last_plotted_=0;
        draw_box();
        axis=last_axis_;        
        fl_clip(_x+_b, _y+_b, _w-2*_b, _h-2*_b);
        while(axis){
            axis->draw_grid();
            axis=axis->previous_axis_;
        }
        fl_pop_clip();
    }
            
    if (last_plotted_)
        last_plotted_=last_plotted_->next;
    else
        last_plotted_=first_object_;
    fl_clip(_x, _y, _w, _h);
    while(last_plotted_){
        last_plotted_->object->draw();
        last_plotted_=last_plotted_->next;
    }
    last_plotted_=last_object_;                
    fl_pop_clip();
    if (damage_&FL_DAMAGE_ALL)
        draw_label();
};
    


void Ca_Canvas::add_object(Ca_Object_ * object){
    last_object_=last_object_->next=new Ca_ObjectChain();
    last_object_->object=object;
};

void Ca_Canvas::clear(){
    while(first_object_)
        delete first_object_->object;
    damage(CA_DAMAGE_ALL);
};

Ca_Canvas::Ca_Canvas(int x, int y, int w, int h, const char *label):Fl_Box(x,y,w,h,label){
    first_object_=last_object_=last_plotted_=0;
    last_axis_=0;
    current_x(0);
    current_y(0);
    border(CANVAS_BORDER);
    current(this);
};

void Ca_Canvas::border(int border){
    border_=border;
    damage(CA_DAMAGE_ALL);
    Ca_Axis_ *axis=last_axis_;
    while(axis){
        axis->damage(CA_DAMAGE_ALL);
        axis=axis->previous_axis_;
    }
};

Ca_Canvas::~Ca_Canvas(){
    clear();
    Ca_Axis_ *axis=last_axis_;
    while(axis){
        last_axis_=axis->previous_axis_;
        delete axis;
        axis = last_axis_;
    }
};
        


///////////////////////////////////////////////////////////////////////////////


int Ca_Lin_Axis_::update(){
    int _min_pos_=min_pos_;
    double _k=k_;
    min_pos_=min_pos();
    max_pos_=max_pos();
    if (min_==max_)
        k_=0;
    else
        k_=(max_pos_-min_pos_)/(max_-min_);
    if((_k!=k_)||(_min_pos_!=min_pos_))
        return 1;
    else
        return 0;
};

void Ca_Lin_Axis_::rescale_move(int when, double  x){
    
    if((when&CA_WHEN_MAX)&&(x>max_)){
        min_ += x-max_;
        max_=x;
        damage(CA_DAMAGE_ALL);
        canvas_->damage(CA_DAMAGE_ALL);
        
    }
    if((when&CA_WHEN_MIN)&&(x<min_)){
        max_ -= min_-x;
        min_=x;
        damage(CA_DAMAGE_ALL);
        canvas_->damage(CA_DAMAGE_ALL);
    }
    valid_=1;
};

int Ca_Lin_Axis_::position(double value){
    if (k_==0) return (min_pos_+max_pos_)/2;
    return (int)(min_pos_+k_*(value-min_));
};

double Ca_Lin_Axis_::value(int pos){
    if (max_==min_)
        return min_;
    return (min_ +(pos-min_pos_)/k_);
};

int Ca_Lin_Axis_::next_tick(int &tick_index, double &tick_value, int &tick_order, double &interval ){
        
//        int tick_number;
//        static double interval_;
        double _tick_interval;
        double minor_number_;
        if (!interval){
            minor_number_= (double)abs(min_pos_-max_pos_)/(double)tick_separation_;
            _tick_interval=tick_interval_;
            if (tick_interval_<0){
                interval=_tick_interval=-_tick_interval;
                tick_order=(int)floor(log10(_tick_interval));
            }else{
                if(_tick_interval!=0){
                    tick_order=(int)floor(log10(fabs(max_-min_)/minor_number_));
                    interval= pow(10,tick_order) * _tick_interval;
                }else
                    for(int i=NO_LIN_DEFAULTS-1;i>=0;i--){
                        tick_order=(int)floor(log10(fabs(max_-min_)/minor_number_));
                        interval= pow(10,tick_order)*(_tick_interval=default_lin_intervals[i]);
                        major_step_=default_lin_major_steps[i];
                        label_step_=default_lin_label_steps[i];
                        if(((max_-min_)/interval)>=minor_number_)
                            break;
                    }
            }
            tick_order;
            tick_value = floor(min()/interval);
            tick_value *= interval;
            tick_index=(int) floor((tick_value /interval)+0.5);
//            tick_number++;
            return 1;
        }else{
            if (tick_value>(max_)){
                tick_index=-1;
                return 0;
            }else{
                tick_value +=interval;
                tick_index++;
//                tick_number++;
                return 1;
            }
        }
    };

Ca_Lin_Axis_::Ca_Lin_Axis_(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label){

};
///////////////////////////////////////////////////////////////////////////////


int Ca_Log_Axis_::update(){
    int _min_pos_=min_pos_;
    double _k=k_;
    double _q=q_;
    min_pos_=min_pos();
    max_pos_=max_pos();
    if (min_==max_)
        k_=0;
    else{
        k_=(max_pos_-min_pos_)/(log(max_)-log(min_));
        q_=min_pos_-k_*log(min_);
    }
    if((_k!=k_)||(_q!=q_))
        return 1;
    else
        return 0;
};

int Ca_Log_Axis_::position(double value){
    if (k_==0) return (min_pos_+max_pos_)/2;
    return (int)(q_+k_*log(value));
};

double Ca_Log_Axis_::value(int pos){
    if (max_==min_)
        return min_;
    return exp(min_ +(pos-q_)/k_);
};

int Ca_Log_Axis_::next_tick(int &tick_index, double &tick_value, int &tick_order, double  &interval_){

        double _tick_interval;
        int minor_number_;

        if (!interval_){
            tick_order=(int)(floor(log10(min_)));
            if (tick_interval_!=0){
                interval_=fabs(tick_interval_);
                number_per_order=(int)floor(10/interval_+0.5);
            }else{
                number_per_order=(int)(abs(min_pos_-max_pos_)/(tick_separation_*log10(max_/min_)));
                if(number_per_order<=1){
                    label_step_=major_step_=3;
                    tick_order = 3*(tick_order/3);
                    interval_=1;
                    number_per_order=0;
                }else{
                    int _no_per_o=number_per_order;
                    for(int i=NO_LOG_DEFAULTS-1;i>=0;i--){
                        major_step_=default_log_major_steps[i];
                        label_step_=default_log_label_steps[i];
                        interval_=default_log_intervals[i];
                        number_per_order=(int)floor(10/interval_+0.5);
                        if((10/interval_)>=_no_per_o)
                            break;
                    }
                }
            }
            tick_index=number_per_order;
            tick_order--;
            tick_value=pow(10,tick_order);
            interval_*=tick_value;
            
            if(!number_per_order){
                tick_order--;
                tick_value /=10;
                tick_index=1;
            }else
                tick_value *=10;
            return 1;
        }else{
            if (tick_value>(max_)){
                tick_index-=1;
                return 0;
            }else{
                if(number_per_order){
                    if(tick_index==number_per_order){
                        tick_order++;
                        interval_*=10;
                        if(number_per_order<10){
                            tick_index=1;
                            tick_value=interval_;
                        }else{
                            tick_index=2;
                            tick_value=2*interval_;
                        }
                    }else{
                        tick_value +=interval_;
                        tick_index++;
                    }
                }else{
                    tick_order++;
                    tick_index++;
                    tick_value *=10;
                }
                return 1;
            }
        }
    };

Ca_Log_Axis_::Ca_Log_Axis_(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label){

};


///////////////////////////////////////////////////////////////////////////////


Ca_LinX_Axis::Ca_LinX_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_X_Axis_(x,y,w,h,label),Ca_Lin_Axis_(x,y,w,h,label){};
Ca_LinRevX_Axis::Ca_LinRevX_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_RevX_Axis_(x,y,w,h),Ca_Lin_Axis_(x,y,w,h){};
Ca_LinY_Axis::Ca_LinY_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_Y_Axis_(x,y,w,h,label),Ca_Lin_Axis_(x,y,w,h,label){};
Ca_LinRevY_Axis::Ca_LinRevY_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_Y_Axis_(x,y,w,h,label),Ca_RevY_Axis_(x,y,w,h),Ca_Lin_Axis_(x,y,w,h){};
Ca_LogX_Axis::Ca_LogX_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_X_Axis_(x,y,w,h,label),Ca_Log_Axis_(x,y,w,h,label){};
Ca_LogRevX_Axis::Ca_LogRevX_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_RevX_Axis_(x,y,w,h,label),Ca_Log_Axis_(x,y,w,h,label){};
Ca_LogY_Axis::Ca_LogY_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_Y_Axis_(x,y,w,h,label),Ca_Log_Axis_(x,y,w,h,label){};
Ca_LogRevY_Axis::Ca_LogRevY_Axis(int x, int y, int w, int h, const char *label):Ca_Axis_(x,y,w,h,label),Ca_Y_Axis_(x,y,w,h,label),Ca_RevY_Axis_(x,y,w,h,label),Ca_Log_Axis_(x,y,w,h,label){};

///////////////////////////////////////////////////////////////////////////////

Ca_Object_::Ca_Object_(Ca_Canvas * canvas){
    canvas_=canvas;
    if(!canvas_)
        canvas_=Ca_Canvas::current();
    Ca_ObjectChain *objectchain=new(Ca_ObjectChain);
    objectchain->object=this;
    objectchain->next=0;
    if(canvas_->last_object_)
        canvas_->last_object_=canvas_->last_object_->next=objectchain;
    else
        canvas_->last_object_=canvas_->first_object_=objectchain;
    x_axis_=canvas_->current_x();
    y_axis_=canvas_->current_y();
    canvas_->damage(CA_DAMAGE_ADD);
};

Ca_Object_::~Ca_Object_(){
    Ca_ObjectChain * objectchain=canvas_->first_object_;
    Ca_ObjectChain * previouschain=0;
    while(objectchain->object!=this){
        previouschain=objectchain;
        objectchain=objectchain->next;
    }
    if(previouschain)
        previouschain->next=objectchain->next;
    else
        canvas_->first_object_=objectchain->next;
    if(canvas_->last_object_==objectchain)
        canvas_->last_object_=previouschain;
    delete objectchain;
    canvas_->last_plotted_=0;
    canvas_->damage(CA_DAMAGE_ALL);
};


    


///////////////////////////////////////////////////////////////////////////////

void Ca_Point::draw(){
    fl_color(color);
    int s,t;
    int _x=x_axis_->position(x);
    int _y=y_axis_->position(y);
    switch(style & POINT_STYLE){
    case SIMPLE:
        fl_point(_x,_y);
        break;
    case ROUND:
        fl_pie(_x-size,_y-size,2*size,2*size,0,360);
        if (style & BORDER){
            fl_color(border_color);
            fl_arc(_x-size,_y-size,2*size,2*size,0,360);
        }
        break;
    case SQUARE:
        fl_rectf(_x-size,_y-size,2*size,2*size);
        if (style & BORDER){
            fl_color(border_color);
            fl_rect(_x-size,_y-size,2*size,2*size);
        }
        break;
    case UP_TRIANGLE:
        s=(int)(1.12*size+0.5);
        t=_y+size/3;
        fl_polygon(_x,t-2*size,_x-s,t,_x+s,t);
        if (style & BORDER){
            fl_color(border_color);
            fl_loop(_x,t-2*size,_x-s,t,_x+s,t);
        }
        break;
    case DOWN_TRIANGLE:
        s=(int)(1.12*size+0.5);
        t=_y-size/3;
        fl_polygon(_x,t+2*size,_x-s,t,_x+s,t);
        if (style & BORDER){
            fl_color(border_color);
            fl_loop(_x,t+2*size,_x-s,t,_x+s,t);
        }
        break;
    case DIAMOND:
        s=(int)(1.3*size+0.5);
        fl_polygon(_x,_y-s,_x-s,_y,_x,_y+s,_x+s,_y);
        if (style & BORDER){
            fl_color(border_color);
            fl_loop(_x,_y-s,_x-s,_y,_x,_y+s,_x+s,_y);
        }
        break;
    }
};

Ca_Point::Ca_Point(double _x, double _y, Fl_Color _color, Fl_Color _border_color, unsigned char _style, int _size):Ca_Object_(0){
    style=_style;
    color=_color;
    border_color=_border_color;
    size=_size;
    x=_x;
    y=_y;
};

////////////////////////////////////////////////////////////////////////////////////
void Ca_LinePoint::draw(){
    Ca_Point::draw();
    if(previous){
        fl_color(color);
        fl_line(previous->x_axis_->position(previous->x),previous->y_axis_->position(previous->y),x_axis_->position(x),y_axis_->position(y));
    }
    
};

Ca_LinePoint::Ca_LinePoint(Ca_LinePoint *_previous, double _x, double _y, Fl_Color color, Fl_Color border_color, unsigned char style, int size):Ca_Point(_x, _y, color, border_color,style, size){
    previous=_previous;
};
Ca_LinePoint::Ca_LinePoint(Ca_LinePoint *_previous, double _x, double _y):Ca_Point(_x, _y, previous->color, previous->border_color,previous->style, previous->size){
    previous=_previous;
};





            







