#include <stdio.h>
#include <math.h>
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include "Cartesian.h"

const double PI=3.1416;
const double MIN_FREQ=90;
const double MAX_FREQ=25000;
const double FREQ_COEF=1.15;

Ca_Axis_*  frequency;
Ca_LinY_Axis* current;
Ca_LogY_Axis* power;
Ca_LinY_Axis* phase;

double R=600;
double L=0.16;
double C=1.6e-7;
double U=1;

Ca_Canvas *canvas;

void next_freq(void *){
    static double f=MIN_FREQ;
    static Ca_LinePoint *P_I=0;
    static Ca_LinePoint *P_P=0;
    static Ca_LinePoint *maximum_power=0;

    double XL=2*PI*f*L;
    double XC=1/(2*PI*f*C);
    double I,P,fi;

    I=U/sqrt(R*R+(XL-XC)*(XL-XC));
    P=I*I*R;
    fi=atan((XL-XC)/R);

    
    
    phase->current();                                //setting coordinate
    new Ca_Point(f,fi,FL_YELLOW,FL_BLACK,DOWN_TRIANGLE|BORDER);current->current();                                //setting coordinate

    current->rescale(CA_WHEN_MAX,I*1100);            //different rescalling for max and min just to get some extra gap 
    current->rescale(CA_WHEN_MIN,I*1000);            //above maximum
    P_I=new Ca_LinePoint(P_I,f,I*1000,FL_BLUE,FL_BLACK,SQUARE|BORDER);

    power->current();                                //setting coordinate
    power->rescale(CA_WHEN_MIN|CA_WHEN_MAX,P*1000);
    P_P=new Ca_LinePoint(P_P,f,P*1000,FL_RED,FL_BLACK,ROUND|BORDER);
    if(maximum_power){
        if(P_P->y > maximum_power->y)                //searching the maximum power
            maximum_power=P_P;
    }else
        maximum_power=P_P;

    f =f* FREQ_COEF;
    if(f<=MAX_FREQ)
        Fl::add_timeout(.5,next_freq);
    else{
        maximum_power->style=(DIAMOND|BORDER);        //changing the shape of the maximum
        canvas->redraw();

    }

}

int main(int argc, char ** argv) {


    Fl_Double_Window *w= new Fl_Double_Window(580, 375, "Cartesian graphics example");

    Fl_Group *c =new Fl_Group(0, 0, 580, 375, "The points are added to the plot in 0.5s intervals by add_timeout().\n You can resize the window to see the changes during or after their generation...");
    c->box(FL_DOWN_BOX);
    c->align(FL_ALIGN_TOP|FL_ALIGN_INSIDE);

    canvas = new Ca_Canvas(180, 75, 300, 225, "RLC resonance circuit");
    canvas->box(FL_DOWN_BOX);
    canvas->color(7);
    canvas->align(FL_ALIGN_TOP_LEFT);
    Fl_Group::current()->resizable(canvas);
    canvas->border(15);

    frequency = new Ca_LogX_Axis(185, 305, 295, 50, "Frequency [Hz]");
    frequency->align(FL_ALIGN_BOTTOM);
    frequency->min(MIN_FREQ);
    frequency->max(MAX_FREQ);
    frequency->label_format("%g");
    frequency->minor_grid_color(fl_gray_ramp(20));
    frequency->major_grid_color(fl_gray_ramp(15));
    frequency->label_grid_color(fl_gray_ramp(10));
    frequency->grid_visible(CA_MINOR_GRID|CA_MAJOR_GRID|CA_LABEL_GRID);
    frequency->major_step(10);
    frequency->label_step(10);
     
    current = new Ca_LinY_Axis(100, 75, 85, 235, "I [mA]");
    current->align(FL_ALIGN_TOP);
    current->major_grid_color(fl_gray_ramp(15));
    current->grid_visible(CA_MAJOR_GRID);
    current->axis_align(CA_LEFT);

    power = new Ca_LogY_Axis(10, 75, 75, 225, "P [mW]");
    power->box(FL_DOWN_BOX);
    power->align(FL_ALIGN_TOP);
    power->min(0.01);                    //setting beggining range
    power->max(1);
    
    phase = new Ca_LinY_Axis(480, 75, 60, 225, "Phase [rad]");
    phase->align(FL_ALIGN_TOP);
    phase->min(-PI);
    phase->max(PI);
    phase->axis_align(CA_RIGHT);
    phase->tick_interval(-PI/4);        //fixed ticks setting
    phase->major_step(2);
    phase->label_step(4);
    phase->label_format("%.2f");
    
    c->end();
    Fl_Group::current()->resizable(c);
    
    w->end();
    w->show();
    Fl::add_timeout(0,next_freq);
 

  Fl::run();
      
  return 0;
}
